#ifndef STRINGTOOLS_H
#define STRINGTOOLS_H

#include <string>

namespace STRING_TOOLS
{
    //------------------------------------------------------------------------
    /**
     * @param s A string to convert to a boolean.
     * @return The bool value of s.
     */
    bool toBool(const std::string &s);

    //------------------------------------------------------------------------
    /**
     * @param s A string to convert to an int.
     * @return The int value of s.
     */
    int toInt(const std::string &s);

    //------------------------------------------------------------------------
    /**
     * @param s A string to convert to an unsigned.
     * @return The unsigned value of s.
     * @todo Negative numbers in s shall lead to an exception.
     */
    unsigned toUnsigned(const std::string &s);

    //------------------------------------------------------------------------
    /**
     * Checks, if the given unsigned value is contained in the token string t.
     *
     * @param t The token string, using ',' as seperation character.
     * @param value The unsigned value to check for.
     *
     * @return true, if value is contained in t, else false.
     */
    bool hasUnsignedInTokenString(const std::string &t, const unsigned value);

    //------------------------------------------------------------------------
    /**
     * Tries to pop the first token of the token string t,
     * which value is equal to the given unsigned value.
     *
     * @param t The token string, using ',' as seperation character.
     * @param value The unsigned value to pop.
     *
     * @return true, if value was popped from t, else false.
     */
    bool popUnsignedFromTokenString(std::string &t, const unsigned value);

    //------------------------------------------------------------------------
    /**
     * Same as popUnsignedFromTokenString(),
     * but only the first token of t is checked.
     *
     * @param t The token string, using ',' as seperation character.
     * @param value The unsigned value to pop.
     *
     * @return true, if value is the first token and was popped from t,
     *         else false.
     */
    bool popFirstUnsignedFromTokenString(std::string &t, const unsigned value);
} // STRING_TOOLS

#endif //STRINGTOOLS_H
